@@ -1,28 +1,15 @@
 //! PKCS#8 `EncryptedPrivateKeyInfo`
 
 use crate::{Error, Result};
-use core::{convert::TryFrom, fmt};
-use der::{
-    asn1::{Any, OctetString},
-    Decodable, Encodable, Message,
-};
+use core::fmt;
+use der::{asn1::OctetString, Decodable, Decoder, Encodable, Sequence};
 use pkcs5::EncryptionScheme;
 
 #[cfg(feature = "alloc")]
 use crate::{EncryptedPrivateKeyDocument, PrivateKeyDocument};
 
-#[cfg(feature = "encryption")]
-use core::convert::TryInto;
-
 #[cfg(feature = "pem")]
-use {
-    crate::{error, pem, LineEnding},
-    zeroize::Zeroizing,
-};
-
-/// Type label for PEM-encoded private keys.
-#[cfg(feature = "pem")]
-pub(crate) const PEM_TYPE_LABEL: &str = "ENCRYPTED PRIVATE KEY";
+use {crate::LineEnding, alloc::string::String, der::Document, zeroize::Zeroizing};
 
 /// PKCS#8 `EncryptedPrivateKeyInfo`.
 ///
@@ -60,51 +47,33 @@ impl<'a> EncryptedPrivateKeyInfo<'a> {
     #[cfg(feature = "encryption")]
     #[cfg_attr(docsrs, doc(cfg(feature = "encryption")))]
     pub fn decrypt(&self, password: impl AsRef<[u8]>) -> Result<PrivateKeyDocument> {
-        self.encryption_algorithm
-            .decrypt(password, &self.encrypted_data)
-            .map_err(|_| Error::Crypto)
-            .and_then(TryInto::try_into)
+        Ok(self
+            .encryption_algorithm
+            .decrypt(password, self.encrypted_data)?
+            .try_into()?)
     }
 
     /// Encode this [`EncryptedPrivateKeyInfo`] as ASN.1 DER.
     #[cfg(feature = "alloc")]
     #[cfg_attr(docsrs, doc(cfg(feature = "alloc")))]
-    pub fn to_der(&self) -> EncryptedPrivateKeyDocument {
-        self.into()
-    }
-
-    /// Encode this [`EncryptedPrivateKeyInfo`] as PEM-encoded ASN.1 DER.
-    #[cfg(feature = "pem")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "pem")))]
-    pub fn to_pem(&self) -> Zeroizing<alloc::string::String> {
-        self.to_pem_with_le(LineEnding::default())
+    pub fn to_der(&self) -> Result<EncryptedPrivateKeyDocument> {
+        self.try_into()
     }
 
     /// Encode this [`EncryptedPrivateKeyInfo`] as PEM-encoded ASN.1 DER with
     /// the given [`LineEnding`].
     #[cfg(feature = "pem")]
     #[cfg_attr(docsrs, doc(cfg(feature = "pem")))]
-    pub fn to_pem_with_le(&self, line_ending: LineEnding) -> Zeroizing<alloc::string::String> {
-        Zeroizing::new(
-            pem::encode_string(PEM_TYPE_LABEL, line_ending, self.to_der().as_ref())
-                .expect(error::PEM_ENCODING_MSG),
-        )
+    pub fn to_pem(&self, line_ending: LineEnding) -> Result<Zeroizing<String>> {
+        Ok(Zeroizing::new(
+            EncryptedPrivateKeyDocument::try_from(self)?.to_pem(line_ending)?,
+        ))
     }
 }
 
-impl<'a> TryFrom<&'a [u8]> for EncryptedPrivateKeyInfo<'a> {
-    type Error = Error;
-
-    fn try_from(bytes: &'a [u8]) -> Result<Self> {
-        Ok(Self::from_der(bytes)?)
-    }
-}
-
-impl<'a> TryFrom<Any<'a>> for EncryptedPrivateKeyInfo<'a> {
-    type Error = der::Error;
-
-    fn try_from(any: Any<'a>) -> der::Result<EncryptedPrivateKeyInfo<'a>> {
-        any.sequence(|decoder| {
+impl<'a> Decodable<'a> for EncryptedPrivateKeyInfo<'a> {
+    fn decode(decoder: &mut Decoder<'a>) -> der::Result<EncryptedPrivateKeyInfo<'a>> {
+        decoder.sequence(|decoder| {
             Ok(Self {
                 encryption_algorithm: decoder.decode()?,
                 encrypted_data: decoder.octet_string()?.as_bytes(),
@@ -113,7 +82,7 @@ impl<'a> TryFrom<Any<'a>> for EncryptedPrivateKeyInfo<'a> {
     }
 }
 
-impl<'a> Message<'a> for EncryptedPrivateKeyInfo<'a> {
+impl<'a> Sequence<'a> for EncryptedPrivateKeyInfo<'a> {
     fn fields<F, T>(&self, f: F) -> der::Result<T>
     where
         F: FnOnce(&[&dyn Encodable]) -> der::Result<T>,
@@ -125,6 +94,14 @@ impl<'a> Message<'a> for EncryptedPrivateKeyInfo<'a> {
     }
 }
 
+impl<'a> TryFrom<&'a [u8]> for EncryptedPrivateKeyInfo<'a> {
+    type Error = Error;
+
+    fn try_from(bytes: &'a [u8]) -> Result<Self> {
+        Ok(Self::from_der(bytes)?)
+    }
+}
+
 impl<'a> fmt::Debug for EncryptedPrivateKeyInfo<'a> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("EncryptedPrivateKeyInfo")
